---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
with X_Types ;

package X_Resource_Manager is
    use X_Types ;
    
    -- The following exception is raised when a given quark is
    -- invalid (not known) by the resource manager
    Quark_Error : exception ;
    
    type Quark is private ;
    subtype Name is Quark ;
    subtype Class is Quark ;
    subtype Representation is Quark ;
    
    Null_Quark : constant Quark ;

    type Quark_Array is array (X_Natural range <>) of Quark ;
    type Quarks is access Quark_Array ;
    type Quark_List ( Length : X_Natural := 0 ) is
    record
        Contents : Quark_Array ( 1 .. Length ) ;
    end record ;
    
    subtype Name_Array  is Quark_Array ;
    subtype Name_List   is Quark_List ;
    subtype Class_Array is Quark_Array ;
    subtype Class_List  is Quark_List ;
    
    type Database is private ;
    
    type Raw_Data is array (Short_Natural range <>) of Byte ;
    
    -- The following type describes a resource value
    -- the Binary discriminant indicates whether the value must be
    -- interpreted as raw data or as a string (default).
    type Resource_Value ( Binary : Boolean := False ;
                          Size   : Short_Natural := 0 ) is
    record
        case Binary is
        when True  => Data  : Raw_Data (1 .. Size) ;
        when False => Chars : String (1 .. Size) ;
        end case ;
    end record ;
    
    type Option_Kind is
        ( No_Arg,      -- Value is specified in Option_Desc_Rec.Value
          Is_Arg,      -- Value is the option string itself		
          Sticky_Arg,  -- Value is characters immediately following option
          Sep_Arg,     -- Value is next argument in argv		
          Res_Arg,     -- Resource and value in next argument in argv
          Skip_Arg,    -- Ignore this option and the next argument in argv
          Skip_Line,   -- Ignore this option and the rest of argv	
          Skip_N_Args  -- Ignore this option and the next 
          ) ;          -- Option_Desc_Record.Value arguments in argv

    type Option_Desc_Record is
    record
        Option    : String_Record  ; -- Option abbreviation in argv
        Specifier : String_Record  ; -- Resource specifier
        Arg_Kind  : Option_Kind    ; -- Which style of option it is
        Value     : Resource_Value ; -- Value to provide if No_Arg
    end record ;
    
    type Option_Desc_Array is array (X_Natural range <>) of
        Option_Desc_Record ;
    type Option_Descs is access Option_Desc_Array ;
    type Option_Desc_List ( Length : X_Short_Natural := 0 ) is
    record
        Contents : Option_Desc_Array ( 1 .. Length ) ;
    end record ;
    

    type General_Binding is ( Bind_Tightly, Bind_Loosely, Is_Representation );
    subtype Binding is General_Binding range Bind_Tightly .. Bind_Loosely ;
    type Binding_Array is array (X_Natural range <>) of General_Binding ;
    type Bindings is access Binding_Array ;
    type Binding_List ( Length : X_Natural := 0 ) is
    record
        Contents : Binding_Array ( 1 .. Length ) ;
    end record ;
    
    -- List type to store pre-fetched database levels
    -- for N components in the searched names or class,
    -- the worst necessary length is 3**N.
    type Search_List (Length : X_Natural) is private ;
    
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Destroy_Database
    --
    --  <Description>
    --
    --  This procedure frees the memory used by the specified database.
    --
    --  <Parameters>
    --
    --  Db     : Specifies the database to destroy.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Destroy_Database ( Db : in out Database ) ;
        
    --======================================================================
    --  <Subprogram Name>
    --
    --  Get_Default
    --
    --  <Description>
    --
    --  This procedure is used to find out the fonts, colors, and other
    --  environment defaults favored by a particular user.
    --
    --  It has been improved (compared to C Xlib) to support a qualified
    --  resource name and not only a single name.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Program        : Specifies the application name
    --  Option         : Specifies the resource for which a value is requested
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Get_Default ( Display_Id : in Display;
                           Program    : in String;
                           Option     : in String) return String;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_File_Database
    --
    --  <Description>
    --
    --  This function creates a database, stores in it the resources
    --  found in the specified text file and returns the created database.
    --
    --  <Parameters>
    --
    --  Name     : Specifies the text file to build the database from.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Get_File_Database ( Name : in String ) return Database ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Resource
    --
    --  <Description>
    --
    --  This procedure gets from the given database a copy of the kind and
    --  the value associated with the given fully qualified name and class
    --  strings.
    --
    --  Null_Quark is return in Kind to indicate that no value was found.
    --
    --  <Parameters>
    --
    --  Db       : Specifies the database to use.
    --  Names    : Specifies the fully qualified name to retrieve
    --  Classes  : Specifies the fully qualified class to retrieve
    --  Kind     : Representation type of the value returned
    --  Value    : Requested value
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Get_Resource ( Db       : in Database ;
                             Names    : in String ;
                             Classes  : in String ;
                             Kind     : out Representation ;
                             Value    : out Resource_Value ) ;
        
    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Resource
    --
    --  <Description>
    --
    --  This procedure gets from the given database a copy of the kind and
    --  the value associated with the given fully qualified name and class
    --  quark lists.
    --
    --  Null_Quark is return in Kind to indicate that no value was found.
    --
    --  <Parameters>
    --
    --  Db       : Specifies the database to use.
    --  Names    : Specifies the fully qualified name to retrieve
    --  Classes  : Specifies the fully qualified class to retrieve
    --  Kind     : Representation type of the value returned
    --  Value    : Requested value
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Get_Resource ( Db       : in Database ;
                             Names    : in Name_Array ;
                             Classes  : in Class_Array ;
                             Kind     : out Representation ;
                             Value    : out Resource_Value ) ;
        
    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Search_List
    --
    --  <Description>
    --
    --  This procedure fills the given search list, in a best to worst order,
    --  with the database levels where a match with the requested name and
    --  class lists provided. If the search list provided is not sufficient
    --  to store all the possible levels, Enough is set to False.
    --
    --  Names and classes must be fully qualified but the last component,
    --  for which values will be got later using Get_Search_Resource.
    --
    --  <Parameters>
    --
    --  Db       : Specifies the database to search.
    --  Names    : Qualifying list of names
    --  Classes  : Qualifying list of classes
    --  List     : Search list to obtain
    --  Enough   : Indicates whether there was room enough to store the levels
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Get_Search_List ( Db       : in Database ;
                                Names    : in Name_Array ;
                                Classes  : in Class_Array ;
                                List     : in out Search_List ;
                                Enough   : out Boolean ) ;
        
    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Search_Resource
    --
    --  <Description>
    --
    --  This procedure gets, using the search list given, a copy of the kind
    --  and the value associated with the given name and class.
    --
    --  Null_Quark is return in Kind to indicate that no value was found.
    --
    --  <Parameters>
    --
    --  List    : Pre-fetched search list
    --  A_Name  : Specify the resource name
    --  A_Class : Specify the resource class
    --  Kind    : Returned representation type
    --  Value   : returned value
    --
    --  <Exceptions Raised>
    --
    --  None.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Get_Search_Resource ( List    : in Search_List ;
                                    A_Name  : in Name ;
                                    A_Class : in Class ;
                                    Kind    : out Representation ;
                                    Value   : out Resource_Value ) ;
        
    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_String_Database
    --
    --  <Description>
    --
    --  This function creates a database, stores in it the resources
    --  found in the specified string and returns the created database.
    --
    --  <Parameters>
    --
    --  Data     : Specifies the string to build the database from.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Get_String_Database ( Data : in String ) return Database ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Initialize
    --
    --  <Description>
    --
    --  This procedure does the necessary initialisations of the resource
    --  manager. Must be called once before any use of the resource manager.
    --
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Initialize ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Merge_Databases
    --
    --  <Description>
    --
    --  This procedure merge a source database into a target database.
    --  Entries in the target may be overwritten.
    --
    --  The source database is destroyed.
    --
    --  <Parameters>
    --
    --  Base     : Specifies the database to insert and then destroy.
    --  Into     : Specifies the target database.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Merge_Databases ( Base : in out Database ;
                                Into : in out Database ) ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  New_Database
    --
    --  <Description>
    --
    --  This function creates a new empty database and returns it.
    --
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function New_Database return Database ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Parse_Command
    --
    --  <Description>
    --
    --  This procedure parse the command list to find the arguments
    --  indicated into Options and to execute  associated actions in
    --  the specified database.
    --
    --  The remaining unused arguments are returned in Command.
    --
    --  <Parameters>
    --
    --  Db      : Specifies the database to update.
    --  Options : Specify how to take command arguments into account
    --  Prefix  : Specify the string to use as resource name prefix
    --  Command : Specify the command to parse.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Parse_Command ( Db      : in out Database ;
                              Options : in Option_Desc_List ;
                              Prefix  : in String ;
                              Command : in out String_List ) ;
        
    --====================================================================
    --  <Subprogram Name>
    --
    --  Put_File_Database
    --
    --  <Description>
    --
    --  This procedure dumps the given database into the indicated text file.
    --
    --  <Parameters>
    --
    --  Db        : Specifies the database to store.
    --  File_Name : Specify the resource file to dump into.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Put_File_Database ( Db        : in Database ;
                                  File_Name : in String ) ;
        
    --====================================================================
    --  <Subprogram Name>
    --
    --  Put_Line_Resource
    --
    --  <Description>
    --
    --  This procedure interprets the string given as resources description
    --  and stores these resources into the given database, which is created
    --  if needed.
    --
    --  <Parameters>
    --
    --  Db     : Specifies the database to update.
    --  Line   : Specifies the text string to interpret.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Put_Line_Resource ( Db   : in out Database ;
                                  Line : in String ) ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Put_Resource
    --
    --  <Description>
    --
    --  This procedure update the given database with the indicated resource
    --  and its associated representation and value.
    --
    --  <Parameters>
    --
    --  Db        : Specifies the database to update.
    --  Specifier : Specifies the fully qualified resource to update.
    --  Kind      : Specifies the representation type of the resource.
    --  Value     : Specifies the value of the resource.
    --
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Put_Resource ( Db        : in out Database ;
                             Specifier : in String ;
                             Kind      : in String ;
                             Value     : in Resource_Value ) ;
        
    --====================================================================
    --  <Subprogram Name>
    --
    --  Put_Resource
    --
    --  <Description>
    --
    --  This procedure update the given database with the indicated resource
    --  and its associated bindings, representation and value.
    --
    --  <Parameters>
    --
    --  Db        : Specifies the database to update.
    --  Bindings  : Specifies the bindings of the resource.
    --  Quarks    : Specifies the resource to update.
    --  Kind      : Specifies the representation type of the resource.
    --  Value     : Specifies the value of the resource.
    --
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Put_Resource ( Db       : in out Database ;
                             Bindings : in Binding_List ;
                             Quarks   : in Quark_List ;
                             Kind     : in Representation ;
                             Value    : in Resource_Value ) ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Put_String_Resource
    --
    --  <Description>
    --
    --  This procedure update the given database with the indicated resource
    --  and its string value.
    --
    --  <Parameters>
    --
    --  Db        : Specifies the database to update.
    --  Specifier : Specifies the fully qualified resource to update.
    --  Value     : Specifies the value of the resource.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Put_String_Resource ( Db        : in out Database ;
                                    Specifier : in String ;
                                    Value     : in String ) ;
        
    --====================================================================
    --  <Subprogram Name>
    --
    --  Put_String_Resource
    --
    --  <Description>
    --
    --  This procedure update the given database with the indicated resource
    --  and its string value.
    --
    --  <Parameters>
    --
    --  Db        : Specifies the database to update.
    --  Bindings  : Specifies the bindings of the resource.
    --  Quarks    : Specifies the resource to update.
    --  Value     : Specifies the value of the resource.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Put_String_Resource ( Db       : in out Database ;
                                    Bindings : in Binding_List ;
                                    Quarks   : in Quark_List ;
                                    Value    : in String ) ;
        
    --====================================================================
    --  <Subprogram Name>
    --
    --  Quark_To_String
    --
    --  Class_To_String, Name_To_String, Representation_To_String
    --
    --  <Description>
    --
    --  This function convert the given quark into the string it represents.
    --
    --  It is renamed to implement conversions from Name,
    --  Class or Representation.
    --
    --  <Parameters>
    --
    --  Quark_Id     : Specifies the quark to convert.
    --
    --  <Exceptions Raised>
    --
    --  Quark_Error.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Quark_To_String ( Quark_Id : in Quark ) return String ;

    function Class_To_String ( Quark_Id : in Class )
        return String renames Quark_To_String ;
    function Name_To_String ( Quark_Id : in Name )
        return String renames Quark_To_String ;
    function Representation_To_String ( Quark_Id : in Representation )
        return String renames Quark_To_String ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  String_Representation
    --
    --  <Description>
    --
    --  This function returns the Representation of type "String".
    --
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function String_Representation return Representation ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  String_To_Binding_Quark_List
    --
    --  <Description>
    --
    --  This procedure converts a fully qualified resource string to its
    --  representation as quark list and list of bindings.
    --
    --  <Parameters>
    --
    --  Item     : Specifies the string to convert.
    --  Bindings : Returned corresponding bindings.
    --  Quarks   : Returned corresponding quark list.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure String_To_Binding_Quark_List
        ( Item     : in String ;
          Bindings : out Binding_List ;
          Quarks   : out Quark_List ) ;
        
    --====================================================================
    --  <Subprogram Name>
    --
    --  String_To_Quark
    --
    --  String_To_Class, String_To_Name, String_To_Representation
    --
    --  <Description>
    --
    --  This function converts a given string into a quark.
    --
    --  It is renamed to implement conversions to Name,
    --  Class or Representation.
    --
    --  <Parameters>
    --
    --  Chars     : Specifies the string to convert.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function String_To_Quark ( Chars : in String ) return Quark ;
    
    function String_To_Class ( Chars : in String )
        return Class renames String_To_Quark ;
    function String_To_Name ( Chars : in String )
        return Name renames String_To_Quark ;
    function String_To_Representation ( Chars : in String )
        return Representation renames String_To_Quark ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  String_To_Quark_List
    --
    --  String_To_Class_List, String_To_Name_List
    --
    --  <Description>
    --
    --  This function converts a given string into a quark.
    --
    --  It is renamed to implement conversions to Name or Class
    --  
    --  <Parameters>
    --
    --  Item     : Specifies the string to convert.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function String_To_Quark_List ( Item : in String ) return Quark_List ;
    
    function String_To_Name_List ( Item : in String )
        return Name_List renames String_To_Quark_List ;
    
    function String_To_Class_List ( Item : in String )
        return Class_List renames String_To_Quark_List ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Unique_Quark
    --
    --  <Description>
    --
    --  This function returns a quark which is guaranteed to represent no
    --  string known by the resource manager.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Unique_Quark return Quark ;
    
private
    type Quark is new X_Natural ;
    Null_Quark : constant Quark := 0 ;
    
    type Hash_Bucket_Record ;
    type Database is access Hash_Bucket_Record ;

    type Search_List (Length : X_Natural) is
    record
        null ;
    end record ;
    
end X_Resource_Manager;
